@@ -6,12 +6,13 @@ This module is a bridge between `jedi.Refactoring` and
 
 
 import difflib
-from typing import Dict, Iterator, List, NamedTuple, Optional, Union
+from typing import Dict, Iterator, List, NamedTuple, Union
 
 from jedi.api.refactoring import ChangedFile, Refactoring
-from pygls.types import (
+from pygls.lsp.types import (
     Position,
     Range,
+    RenameFile,
     RenameFileOptions,
     TextDocumentEdit,
     TextEdit,
@@ -20,22 +21,6 @@ from pygls.types import (
 from pygls.workspace import Workspace
 
 
-class RenameFile:  # pylint: disable=too-few-public-methods
-    """Pygls has bug right now, this would be simple pull request."""
-
-    def __init__(
-        self,
-        old_uri: str,
-        new_uri: str,
-        options: Optional[RenameFileOptions] = None,
-    ):
-        # pylint: disable=invalid-name
-        self.kind = "rename"
-        self.oldUri = old_uri
-        self.newUri = new_uri
-        self.options = options
-
-
 def lsp_document_changes(
     workspace: Workspace,
     refactoring: Refactoring,
